import requests
from lxml import html
from requests import RequestException


class Investing:
    def __init__(self, url: str, name: str = None):
        self.url = url
        self.name = name
        self.page = None
        self.ok = None
        self.__user_agent = "InvestingCrawler/1.0"
        self.__headers = {"User-Agent": self.__user_agent}

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value: str) -> str:
        if value:
            self._name = value
        else:
            self._name = self.url.split("/")[-1]
        return self._name

    @property
    def url(self) -> str:
        return self._url

    @url.setter
    def url(self, value: str) -> str:
        if not value:
            raise ValueError("Value cannot be empty.")
        else:
            self._url = "https://www.investing.com/" + value
        return self._url

    def fetch(self) -> None:
        """Get webpage as defined by url attribute.

        Args:
            None

        Returns:
            page (lxml.htmlElement): Content of the url.

        """
        response = requests.get(self.url, headers=self.__headers)
        self.ok = response.ok

        if response.ok:
            self.page = html.fromstring(response.content)
        else:
            raise RequestException(f"Got {response.status_code}")

    def price(self) -> float:
        """Return the value(price) set by span with @id last_last.

        Args:
            None

        Returns:
            _price (float): Price as present in the span.

        """
        if len(self.page):
            _price_str = str(self.page.xpath("//span[@id='last_last']/text()")[0])
            _price = float(_price_str.replace(",", ""))
            return _price
        else:
            raise ValueError(f"No page found or length of page is {len(self.page)}")



import logging
import sys
import time

import click
from investing import Investing
from sendtext import SendText

__version__ = "0.0.3"


def run(
    instrument: Investing,
    send_message: SendText,
    lower: float,
    upper: float,
    threshold: float,
) -> tuple:
    instrument.fetch()
    logger.debug("Fetched page successfully.")

    price = instrument.price()
    logger.debug(f"Price of {instrument.name} is ${price}.")

    if price >= upper or price <= lower:
        logger.info(f"Price {price} breached price band [{lower}, {upper}].")
        logger.debug(f"Resetting price band with threshold value {threshold}.")
        upper = price * (1 + threshold / 10000)
        lower = price * (1 - threshold / 10000)
        logger.info(f"Resetting price band to [{lower}, {upper}].")
        logger.debug("Sending text.")
        send_message.send(f"{instrument.name} price is {price}.")

    return (lower, upper)


@click.command(
    context_settings=dict(help_option_names=["-h", "--help"]),
    options_metavar="[options...]",
)
@click.argument("to_num", metavar="[to number]")
@click.argument("from_num", metavar="[from number]")
@click.argument("market", metavar="[market]")
@click.argument("contract", metavar="[contract]")
@click.argument("priceband", metavar="[priceband]")
@click.option("--symbol", "-s", help="Contract symbol. [default: contract]")
@click.option(
    "--threshold", "-t", help="Threshold in bps.", default=100.0, show_default=True
)
@click.option(
    "--interval",
    "-i",
    help="Interval to perform check (mins).",
    default=1.0,
    show_default=True,
)
@click.option(
    "--sub-market", "-m", help="E.g. crypto is market and bitcoin is sub market."
)
@click.option("--debug", "-d", is_flag=True, help="Print debug messages.")
def main(
    to_num,
    from_num,
    interval,
    threshold,
    symbol=None,
    market=None,
    contract=None,
    priceband=None,
    sub_market=None,
    debug=None,
):
    """Utiltiy script to notify if instrument price fluctuates out of price band.
    """
    if debug:
        logger.setLevel(logging.DEBUG)
        logger.debug("Logging set to debug")

    lower, upper = list(map(float, priceband.split("-")))

    if sub_market:
        end_point = market + "/" + sub_market + "/" + contract
    else:
        end_point = market + "/" + contract

    logger.debug(f"{end_point} end point will be queried.")

    instrument = Investing(end_point, symbol)
    text_client = SendText(from_num, to_num)

    while True:
        try:
            lower, upper = run(instrument, text_client, lower, upper, threshold)
            time.sleep(60 * interval)
        except KeyboardInterrupt:
            logger.info("Caught interrupt, exiting...")
            sys.exit()


if __name__ == "__main__":
    log_handler = logging.StreamHandler()
    log_handler.setFormatter(
        logging.Formatter(
            "%(asctime)s [%(levelname)5s] %(funcName)4s() - %(message)s",
            "%Y-%m-%d %H:%M:%S",
        )
    )
    logger = logging.getLogger(__name__)
    logger.addHandler(log_handler)
    logger.setLevel(logging.INFO)

    main()



from twilio.rest import Client


class SendText:
    def __init__(self, from_number, to_number):
        self.client = Client()
        self.from_wsp_number = from_number
        self.to_wsp_number = to_number

    @property
    def from_wsp_number(self):
        return self._from_wsp_number

    @property
    def to_wsp_number(self):
        return self._to_wsp_number

    @from_wsp_number.setter
    def from_wsp_number(self, value: str):
        if value:
            self._from_wsp_number = "whatsapp:" + value
        else:
            raise ValueError("From phone number cannot be empty.")

    @to_wsp_number.setter
    def to_wsp_number(self, value: str):
        if value:
            self._to_wsp_number = "whatsapp:" + value
        else:
            raise ValueError("To phone number cannot be empty.")

    def send(self, message: str):
        self.client.messages.create(
            body=message, from_=self.from_wsp_number, to=self.to_wsp_number
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['investify']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.1,<8.0.0',
 'lxml>=4.5.0,<5.0.0',
 'requests>=2.23.0,<3.0.0',
 'twilio>=6.38.0,<7.0.0']

entry_points = \
{'console_scripts': ['investify = investify.investify:main']}

setup_kwargs = {
    'name': 'investify',
    'version': '0.0.3',
    'description': 'Get automated price alert based on investing.com prices',
    'long_description': "# investify\n\nUtility tool which scrapes investing.com website for price updates and send whatsapp notification if it breaches a predefined threshold.\n\n\n## Setup\n\nYou can clone the project and simply run the script. You need to have a [twilio](https://www.twilio.com/) account (a free one would do as well). Setup your whatsapp dashboard in twilio first and use that numbers in the script. For the authentication export your Account SSID and Authentication token as follows :-\n\nAccount SSID\n\n    $ export TWILIO_ACCOUNT_SID='ACXXXXXXXXXXXXXXXXXXXXXXXXXXX'\n\nAuthentication token\n\n    $ export TWILIO_AUTH_TOKEN='XXXXXXXXXXXXXXXXXXXXXXXXX'\n\n\n## Usage\n\n```bash\n$ python investify.py +1456789012 +31234567789 crypto btc-usd 6600-6700 --sub-market bitcoin --threshold 10\n```\nThis will notify when btc-usd prices breaches the price band of 6600-6700. It currently resets the price band from the last breached price.\n\nThe tool will logging INFO (or DEBUG) level messages in standard out. This can certainly be redirected to a specified log file.\n\nDebug level messages as follows :-\n```bash\n$ python investify.py --debug +11234567890 +7112356789 crypto btc-usd 6600-6700 --sub-market bitcoin --threshold 10 --interval 0.1\n2020-04-05 20:13:38 [DEBUG] main() - Logging set to debug\n2020-04-05 20:13:38 [DEBUG] main() - crypto/bitcoin/btc-usd end point will be queried.\n2020-04-05 20:13:39 [DEBUG]  run() - Fetched page successfully.\n2020-04-05 20:13:39 [DEBUG]  run() - Price of btc-usd is $6804.9.\n2020-04-05 20:13:39 [ INFO]  run() - Price 6804.9 breached price band [6600.0, 6700.0].\n2020-04-05 20:13:39 [DEBUG]  run() - Resetting price band with threshold value 10.0.\n2020-04-05 20:13:39 [ INFO]  run() - Resetting price band to [6798.0951, 6811.704899999999].\n2020-04-05 20:13:39 [DEBUG]  run() - Sending text.\n2020-04-05 20:13:46 [DEBUG]  run() - Fetched page successfully.\n2020-04-05 20:13:46 [DEBUG]  run() - Price of btc-usd is $6803.6.\n^C\n2020-04-05 20:13:49 [ INFO] main() - Caught interrupt, exiting...\n```\n\nHelp menu for the tool.\n```bash\n$ python investify.py -h\nUsage: investify.py [options...] [to number] [from number] [market] [contract]\n                    [priceband]\n\n  Utiltiy script to notify if instrument price fluctuates out of price band.\n\nOptions:\n  -s, --symbol TEXT      Contract symbol. [default: contract]\n  -t, --threshold FLOAT  Threshold in bps.  [default: 100.0]\n  -i, --interval FLOAT   Interval to perform check (mins).  [default: 1.0]\n  -m, --sub-market TEXT  E.g. crypto is market and bitcoin is sub market.\n  -d, --debug            Print debug messages.\n  -h, --help             Show this message and exit.\n```\n\n`to number` should be your whatsapp number whereas `from number` should the number given by Twilio to be used for your whatsapp sandbox. You can consult official Twilio for whatsapp [page](https://www.twilio.com/docs/whatsapp/api).\n",
    'author': 'Abhishek Guha',
    'author_email': 'abhi.workspace@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/abhi-g80/investify',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
